"""
Represents an item list contained within Repository
keeps the set of methods executed for add/edit/delete
"""


class Memory:

    def __init__(self, on_add, on_edit, on_remove, items):
        self.on_add = on_add
        self.on_remove = on_remove
        self.on_edit = on_edit
        self.items = items


from cryptomodel.cryptostore import user_transaction, OPERATIONS

from cryptodataaccess.Memory import Memory
from cryptodataaccess.Repository import Repository


class TransactionRepository(Repository):

    def __init__(self, transaction_store):
        self.transaction_store = transaction_store
        self.transactions = []
        super(TransactionRepository, self).__init__()
        memory = Memory(on_add=self.transaction_store.insert_transaction,
                        on_edit=self.transaction_store.update_transaction,
                        on_remove=self.transaction_store.delete_transaction,
                        items=self.transactions
                        )
        self.memories.append(memory)

    def get_distinct_user_ids(self):
        return self.transaction_store.fetch_distinct_user_ids()

    def get_transaction(self, id):
        return self.transaction_store.fetch_transaction(id)

    def get_transactions(self, user_id):
        return self.transaction_store.fetch_transactions(user_id)

    def add_transaction(self, user_id, volume, symbol, value, price, currency, date, source, source_id):
        t = user_transaction(user_id=user_id, volume=volume, symbol=symbol, value=value, price=price,
                             currency=currency, date=date, source=source, source_id=source_id,
                             operation=OPERATIONS.ADDED.name)
        self.transactions.append(t)
        return t

    def edit_transaction(self, in_id, user_id, volume, symbol, value, price, currency, date, source, source_id):
        t = user_transaction(id=in_id,
                             user_id=user_id, volume=volume, symbol=symbol, value=value, price=price,
                             currency=currency, date=date, source=source, source_id=source_id,
                             operation=OPERATIONS.MODIFIED.name)
        self.transactions.append(t)
        return t

    def remove_transaction(self, in_id):
        trans = next((x for x in self.transactions if x.id == in_id), None)
        self.mark_deleted(trans)

    def remove_transaction_by_source_id(self, source_id, throw_if_does_not_exist=True):
        trans = next((x for x in self.transactions if x.source_id == source_id), None)
        self.mark_deleted(trans)

    # reuse across repos
    def mark_deleted(self, trans):
        if trans is None:
            trans = self.get_transaction(trans.id)
            if trans is None:
                exit
            else:
                trans.operation = OPERATIONS.REMOVED.name
                self.transactions.append(trans)
        else:
            trans.operation = OPERATIONS.REMOVED.name



import mock
from cryptomodel.coinmarket import  prices
from mongoengine import  Q
from cryptodataaccess.config import configure_app
import pytest
from cryptodataaccess.helpers import do_connect
from tests.helpers import insert_prices_record


@pytest.fixture(scope='module')
def mock_log():
    with mock.patch("cryptodataaccess.helpers.log_error"
                    ) as _mock:
        _mock.return_value = True
        yield _mock


def test_eval_collection():
    config = configure_app()
    do_connect(config)
    actual_count = prices.objects()
    j = "prices"
    assert(len(eval(j).objects()) == len(actual_count))


def test_get_latest_prices_by_date_eval():
    config = configure_app()
    do_connect(config)
    actual_count = prices.objects()
    j = "prices"
    prices.objects.all().delete()
    insert_prices_record()
    exec("before_date = '2021-01-01'")
    assert (len( eval("prices.objects(Q(status__timestamp__lte=before_date)).order_by('-status__timestamp')[:1]")) == 1)

    eval("prices.objects(Q(status__timestamp__lte=before_date)).order_by('-status__timestamp')[:1]")



from cryptomodel.operations import OPERATIONS


class Repository:

    def __init__(self):
        self.memories = []

    def commit(self):
        for mem in self.memories:
            for item in mem.items:
                if item.operation == OPERATIONS.ADDED.name:
                    trans = mem.on_add(item)
                    if trans is not None:
                        item.id = trans.id
                elif item.operation == OPERATIONS.MODIFIED.name:
                    trans = mem.on_edit(item)
                    if trans is not None:
                        item.id = trans.id
                elif item.operation == OPERATIONS.REMOVED.name:
                    mem.on_remove(item, False)



